<?php

namespace App\Controllers;

require '../config/env.php';

use App\Models\Location;
use App\Models\Menu;
use App\Models\Subscriber;
use App\Models\User;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use App\Models\Payment;

class SubscriptionController
{
    public function createSubscription(Request $request, Response $response, $args)
    {
        // Response helper function
        $responseData = function ($msg, bool $is_success) use ($response): Response {
            $response->getBody()->write(json_encode([
                "status" => $is_success ? true : false,
                "message" => $msg,
                "response_code" => $is_success ? 200 : 400,
            ]));

            return $response;
        };

        // Input data validation
        $contents = json_decode(file_get_contents('php://input'), true);

        if (json_last_error() === JSON_ERROR_NONE) {
            $input_data = $contents;
        } else {
            return $responseData('Invalid JSON', false);
        }

        $params = [
            'client_path' => $input_data['client_path'] ?? null,
            'name' => $input_data['name'] ?? null,
            'email' => $input_data['email'] ?? null,
            'plan_id' => $input_data['plan_id'] ?? 1,
            'payment_method' => $input_data['payment_method'] ?? null,
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            return $responseData(implode(', ', $errors), false);
        }

        // Get client id using b2c path
        $client = (new Menu())->getClientUsingB2cPath($params['client_path']);

        if (!isset($client->id)) {
            return $responseData("Invalid Client Path", false);
        }

        // Get user by mail address
        $user = (new User())->checkCustomerByEmail($params['email']);

        if (empty($user)) {
            $lastInsertId = (new User())->createCustomerByEmail($params['email'], $params['name'], "");
            (new User())->createUserByEmail($params['email'], $lastInsertId->insert_id, 0, "");

            # get user detail
            $user = (new User())->checkCustomerByEmail($params['email']);
        }

        $subscriber = new Subscriber($client->id, $user->id, $params['plan_id']);

        // Check if user already subscribed
        if ($subscriber->status !== null) {
            if ($subscriber->is_subscribed()) {
                return $responseData('User already subscribed', false);
            }
        }

        if (!isset($params['payment_method']) || $params['payment_method'] == "") {
            return $responseData("Payment method is missing", false);
        }

        # plan details
        $planDetail = $subscriber->getPlanDetails();

        if ($planDetail === false) {
            return $responseData("Invalid Plan ID", false);
        }

        $payment = new Payment();
        $stripeConnection = (new menu())->getPaymentDetail($client->id);

        $priceStripeId = '';

        # create price
        if (empty($planDetail->stripe_id) || $stripeConnection->live == 0) {
            // Get currency info
            $location = new Location($client->id);
            $currency = $location->getCurrency();

            $createStripePrice = $payment->createStripePrice($stripeConnection->securitykey, str_replace(".", "", $planDetail->amount), $planDetail->type, $planDetail->duration, $planDetail->plan, 'inclusive', strtolower($currency['code']));

            if ($createStripePrice['status'] == false) {
                return $responseData($createStripePrice['data'], false);
            }

            $priceStripeId = $createStripePrice['data']['id'];

            if ($stripeConnection->live) {
                // update stripe_id in plan detail
                $subscriber->updatePlanDetails(['stripe_id' => $createStripePrice['data']['id']]);
            }
        } else {
            $priceStripeId = $planDetail->stripe_id;
        }

        # create customer
        $createStripeCustomer = $payment->createStripeCustomer($stripeConnection->securitykey, $client->id . "-" . str_replace(' ', '_', $user->fname . '_' . $user->lname) . "-" . $user->email, $user->email, $params['payment_method'], $client->id);

        if (isset($createStripeCustomer['status']) && $createStripeCustomer['status'] == false) {
            return $responseData($createStripeCustomer['data'], false);
        }

        if ($stripeConnection->live) {
            $tax_rate_key = $_ENV['SUBSCRIPTION_VAT_LIVE_KEY'];
        } else {
            $tax_rate_key = $_ENV['SUBSCRIPTION_VAT_TEST_KEY'];
        }

        # calculate the client transfer amount
        $clientTransferPct = 100;

        if($stripeConnection->stripecharge) {
            $stripeCharge = $planDetail->amount * ($stripeConnection->stripecharge / 100);
            if($stripeConnection->stripeadd) {
                $stripeCharge += $stripeConnection->stripeadd;
            }

            $clientTransferPct = round((1 - ($stripeCharge / $planDetail->amount)) * 100, 2);
        }

        # create subscription
        $createSubscription = $payment->createSubscription($stripeConnection->securitykey, $createStripeCustomer->id, $priceStripeId, $clientTransferPct, $createStripeCustomer->default_source, $stripeConnection->connectid, $tax_rate_key, $client->id);

        if ($createSubscription['status'] == false) {
            return $responseData($createSubscription['data'], false);
        }

        $stripeSubscription = $createSubscription['data'];

        # insert subscription table
        $data['user_id'] = $user->id;
        $data['amount'] = $stripeSubscription->items->data[0]->plan->amount / 100;
        $data['vat'] = $stripeSubscription->latest_invoice->tax / 100;
        $data['currency'] = $stripeSubscription->currency;
        $data['duration_type'] = $stripeSubscription->plan->interval;
        $data['duration_count'] = $stripeSubscription->plan->interval_count;
        $data['customer_id'] = $stripeSubscription->customer;
        $data['plan_id'] = $stripeSubscription->plan->id;
        $data['product_id'] = $stripeSubscription->plan->product;
        $data['payment_method'] = $params['payment_method'];
        $data['client_secret'] = $stripeSubscription->latest_invoice->payment_intent->client_secret;
        $data['payment_intent'] = $stripeSubscription->latest_invoice->payment_intent->id;
        $data['subscription_id'] = $stripeSubscription->id;
        $data['subscription_status'] = $stripeSubscription->status;
        $data['is_live'] = $stripeSubscription->livemode;

        # pay invoice
        $payInvoice = $payment->payInvoice($stripeConnection->securitykey, $stripeSubscription->latest_invoice->id);

        if ($payInvoice['status'] == false) {
            return $responseData($payInvoice['data'], false);
        }

        # create subscriber
        $subscriber->createSubscriber();
        $createFKSubscriberDetails = $subscriber->createSubscriberDetails($data);

        $transferdAmount = $payInvoice['data']->total / 100;
        $content['amount'] = number_format($transferdAmount, 2);
        $content['plan'] = $planDetail->plan;
        $content['duration'] = $planDetail->type;

        return $responseData($content, true);
    }

    public function getSubscriptionCoupon(Request $request, Response $response, $args)
    {
        // Response helper function
        $responseData = function ($msg, bool $is_success) use ($response): Response {
            $response->getBody()->write(json_encode([
                "status" => $is_success ? true : false,
                "message" => $msg,
                "response_code" => $is_success ? 200 : 400,
            ]));

            return $response;
        };

        // Input data validation
        $contents = json_decode(file_get_contents('php://input'), true);

        if (json_last_error() === JSON_ERROR_NONE) {
            $input_data = $contents;
        } else {
            return $responseData('Invalid JSON', false);
        }

        $params = [
            'client_path' => $input_data['client_path'] ?? null,
            'user_name' => $input_data['user_name'] ?? null,
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            return $responseData(implode(', ', $errors), false);
        }

        // Get client id using b2c path
        $client = (new Menu())->getClientUsingB2cPath($params['client_path']);

        if (!isset($client->id)) {
            return $responseData("Invalid Client Path", false);
        }

        if (filter_var($params['user_name'], FILTER_VALIDATE_EMAIL)) {
            // Get user by mail address
            $user = (new User())->checkCustomerByEmail($params['user_name']);

            if (empty($user)) {
                return $responseData("Invalide email", false);
            }
        } elseif (is_numeric($params['user_name'])) {
            $user = (new User())->checkCustomerByMobile($params['user_name']);

            if (empty($user)) {
                return $responseData("Invalide phone number", false);
            }
        }

        $subscriber = new Subscriber($client->id, $user->id, 1);
        // Check if user already subscribed
        if ($subscriber->status !== null) {
            if ($subscriber->is_subscribed()) {
                return $responseData([
                    'coupons' => $subscriber->getCouponCodes(),
                ], true);
            }
        }

        return $responseData('User not subscribed', false);
    }

    public function unSubscription(Request $request, Response $response, $args)
    {
        // Response helper function
        $responseData = function ($msg, bool $is_success) use ($response): Response {
            $response->getBody()->write(json_encode([
                "status" => $is_success ? true : false,
                "message" => $msg,
                "response_code" => $is_success ? 200 : 400,
            ]));

            return $response;
        };

        // Input data validation
        $contents = json_decode(file_get_contents('php://input'), true);

        if (json_last_error() === JSON_ERROR_NONE) {
            $input_data = $contents;
        } else {
            return $responseData('Invalid JSON', false);
        }

        $params = [
            'client_path' => $input_data['client_path'] ?? null,
            'user_name' => $input_data['user_name'] ?? null,
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            return $responseData(implode(', ', $errors), false);
        }

        // Get client id using b2c path
        $client = (new Menu())->getClientUsingB2cPath($params['client_path']);

        if (!isset($client->id)) {
            return $responseData("Invalid Client Path", false);
        }

        if (filter_var($params['user_name'], FILTER_VALIDATE_EMAIL)) {
            // Get user by mail address
            $user = (new User())->checkCustomerByEmail($params['user_name']);

            if (empty($user)) {
                return $responseData("Invalide email", false);
            }
        } elseif (is_numeric($params['user_name'])) {
            $user = (new User())->checkCustomerByMobile($params['user_name']);

            if (empty($user)) {
                return $responseData("Invalide phone number", false);
            }
        }

        $subscriber = new Subscriber($client->id, $user->id, 1);
        // Check if user already subscribed
        if ($subscriber->status !== null) {
            if ($subscriber->is_subscribed()) {
                if($subscriber->is_unsubscribe()) {
                    return $responseData('User already unsubscribed.', false);
                }

                $data = $subscriber->getSubscription();
                if ($data) {
                    $stripeConnection = (new menu())->getPaymentDetail($client->id);

                    $payment = new Payment();
                    $result = $payment->unSubscripe($stripeConnection->securitykey, $data['subscription_id']);

                    if($result['status']) {
                        $subscriber->unsubscribe($data['subscription_id'], $result['data']->cancel_at);

                        return $responseData('Successfully unsubscribed.', true);
                    }
                }

                return $responseData('Failed to unsubscribe.', false);
            }
        }

        return $responseData('User not subscribed', false);
    }
}