<?php

ini_set('display_errors', 1);
error_reporting(E_ALL);
date_default_timezone_set('Europe/London');

use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use Slim\Factory\AppFactory;

require __DIR__ . '/../vendor/autoload.php';
$app = AppFactory::create();

$app->add(function (Request $request, $handler) {
    
    $defaultRateLimit = 30;
    $defaultTimeWindow = 60;

    $specificRateLimit = 30;
    $specificTimeWindow = 60;
    

    $path = $request->getUri()->getPath();
    $clientIdentifier = $request->getServerParams()['REMOTE_ADDR'];

   $throttleDataFile = __DIR__ . '/../storage/throttle.json';

    if (file_exists($throttleDataFile)) {
        $throttleData = json_decode(file_get_contents($throttleDataFile), true);
    } else {
        $throttleData = [];
    }

    $currentTime = time();

    if ($path === '/getRestaurantDatas') {
        $rateLimit = $specificRateLimit;
        $timeWindow = $specificTimeWindow;
    } else if($path === '/getMenuPage'){
        $rateLimit = $specificRateLimit;
        $timeWindow = $specificTimeWindow;
    }else {
        $rateLimit = $defaultRateLimit;
        $timeWindow = $defaultTimeWindow;
    }

    if(!empty($throttleData) && is_array($throttleData)){
        if(count($throttleData)>0)
        {
            foreach ($throttleData as $key => $data) {
                if ($currentTime - $data['start_time'] > $timeWindow) {
                    unset($throttleData[$key]);
                }
            }
        }
    }

    $clientKey = $clientIdentifier . '|' . $path;

    if (!isset($throttleData[$clientKey])) {
        $throttleData[$clientKey] = [
            'start_time' => $currentTime,
            'request_count' => 0,
            'time_window' => $timeWindow,
        ];
    }
    $throttleData[$clientKey]['request_count']++;

    if ($throttleData[$clientKey]['request_count'] > $rateLimit) {
        $response = new \Slim\Psr7\Response();
        $response->getBody()->write(json_encode([
            'error' => 'Too Many Requests',
            'message' => 'Rate limit exceeded. Please try again later.',
        ]));
        return $response
            ->withHeader('Content-Type', 'application/json')
            ->withHeader('X-RateLimit-Limit', $rateLimit)
            ->withHeader('X-RateLimit-Remaining', 0)
            ->withHeader('X-RateLimit-Reset', $timeWindow)
            ->withStatus(429);
    }
    file_put_contents($throttleDataFile, json_encode($throttleData));
    $response = $handler->handle($request);
    $remainingRequests = max(0, $rateLimit - $throttleData[$clientKey]['request_count']);
    $response = $response
        ->withHeader('X-RateLimit-Limit', $rateLimit)
        ->withHeader('X-RateLimit-Remaining', $remainingRequests)
        ->withHeader('X-RateLimit-Reset', $timeWindow);
    return $response;
});


$app->add(function ($request, $handler) {
	if ($request->getMethod() === 'OPTIONS') {
		$response = new \Slim\Psr7\Response();
		$response = $response->withHeader('Access-Control-Allow-Origin', '*')
			->withHeader('Access-Control-Allow-Methods', 'POST')
			->withHeader('Access-Control-Allow-Headers', 'Content-Type');
		return $response;
	}

	$response = $handler->handle($request);
	$response = $response->withStatus(200)
		->withHeader('Access-Control-Allow-Origin', '*')
		->withHeader('Content-Type', 'application/json');
	return $response;
});

// error middleware
$errorMiddleware = $app->addErrorMiddleware(true, true, true);
$errorMiddleware->setDefaultErrorHandler(function ($request, $exception, $displayErrorDetails) {
	// Define the log directory
	$logDirectory = __DIR__ . '/../logs/';

	// Create the log directory if it doesn't exist
	if (!file_exists($logDirectory)) {
		mkdir($logDirectory, 0777, true); // Create the directory recursively
	}

	// Check if the directory exists or was created successfully
	if (is_dir($logDirectory) && is_writable($logDirectory)) {
		// Create a unique log file based on timestamp
		$logFile = $logDirectory . 'error_' . date('Y-m-d') . '.log';

		// Log the exception to the dynamically created log file
		$logMessage = '[' . date('Y-m-d H:i:s') . '] ' . $exception . PHP_EOL;
		file_put_contents($logFile, $logMessage, FILE_APPEND | LOCK_EX);

		// Handle the exception and return a response
		$statusCode = $exception->getCode() ?: 500;
		$errorMessage = ($displayErrorDetails) ? $exception->getMessage() : 'An error occurred.';
		$response = new \Slim\Psr7\Response();
		$response->getBody()->write("Error: " . $errorMessage);
		return $response->withStatus($statusCode);
	} else {
		$response = new \Slim\Psr7\Response();
		$response->getBody()->write("Error: Log directory is not accessible.");
		return $response->withStatus(500);
	}
});

# empty response
$app->get('/', function (Request $request, Response $response, $args) {
	$response->getBody()->write("Welcome to FK");
	return $response;
});
$app->post('/', function (Request $request, Response $response, $args) {
	$response->getBody()->write("Welcome to FK");
	return $response;
});

# home page api
$app->get('/homePage', 'App\Controllers\HomeController:index');
$app->post('/getAppDownloadLink', 'App\Controllers\HomeController:getAppDownloadLink');
$app->post('/websocket', 'App\Controllers\HomeController:websocketTrigger');
$app->get('/unsubscribe', 'App\Controllers\HomeController:unsubscribe');
$app->post('/emailwebhook', 'App\Controllers\HomeController:emailwebhook');

# restaurant page api
$app->post('/getRestaurantDatas', 'App\Controllers\RestaurantController:getRestaurantDatas');
$app->post('/getRestaurantFiltersList', 'App\Controllers\RestaurantController:getRestaurantFiltersList');
$app->post('/getCuisinesList', 'App\Controllers\RestaurantController:getCuisinesList');

$app->post('/getRestaurant', 'App\Controllers\RestaurantController:getRestaurant');
$app->post('/favouriteAction', 'App\Controllers\RestaurantController:favouriteAction');
$app->post('/noTakeaway', 'App\Controllers\RestaurantController:noTakeaway');

# menu page api
$app->post('/getRestaurantMenuDetail', 'App\Controllers\MenuController:getRestaurantMenuDetailOptimize');
$app->post('/getRestaurantMenuDetailOptimize', 'App\Controllers\MenuController:getRestaurantMenuDetailOptimize');
$app->post('/getMenuPage', 'App\Controllers\MenuController:getMenuPage');
$app->post('/getReviewPage', 'App\Controllers\MenuController:getReviewPage');
$app->post('/getAboutPage', 'App\Controllers\MenuController:getAboutPage');
$app->post('/getOrderTime', 'App\Controllers\MenuController:getOrderTime');
$app->post('/getItemImage', 'App\Controllers\MenuController:getItemImage');
$app->post('/getItemAddonList', 'App\Controllers\MenuController:getItemAddonList');
$app->post('/checkDeliveryPostcode', 'App\Controllers\PostcodeController:checkDeliveryPostcode');
$app->post('/checkVoucherCode', 'App\Controllers\MenuController:checkVoucherCode');
$app->post('/getItemsNotAddOns', 'App\Controllers\MenuController:getItemsNotAddOns');

# user login api
$app->post('/loginOtpGenerate', 'App\Controllers\UserController:loginOtpGenerate');
$app->post('/resendOtp', 'App\Controllers\UserController:resendOtp');
$app->post('/ssoLoginRegister', 'App\Controllers\UserController:ssoLoginRegister');

# static page api
$app->post('/getHelpPageList', 'App\Controllers\PagesController:getHelpPageList');
$app->post('/resellerSignup', 'App\Controllers\PagesController:resellerSignup');
$app->post('/contactlessDinningSignup', 'App\Controllers\PagesController:contactlessDinning');
$app->post('/careerForm', 'App\Controllers\PagesController:careerForm');
$app->post('/partnerSignup', 'App\Controllers\PagesController:partnerSignup');

# checkout page api
$app->post('/getChargesDetail', 'App\Controllers\MenuController:getChargesDetail');
$app->post('/getStripePaymentDetail', 'App\Controllers\PaymentController:getStripePaymentDetail');
$app->post('/placeOrderWithPayment', 'App\Controllers\PaymentController:placeOrderWithPayment');
$app->post('/updateCustomerAddressPopup', 'App\Controllers\MyAccountController:updateCustomerAddressPopup');
$app->post('/getLoyaltyPoints', 'App\Controllers\PaymentController:getLoyaltyPointsWithOrderAmount');
$app->post('/itemAvailable', 'App\Controllers\MenuController:getItemAvailable');

# myAccount api
$app->post('/getProfile', 'App\Controllers\MyAccountController:getProfile');
$app->post('/sendProfileOtp', 'App\Controllers\MyAccountController:sendProfileOtp');
$app->post('/updateProfile', 'App\Controllers\MyAccountController:updateProfile');
$app->post('/getAddress', 'App\Controllers\MyAccountController:getAddress');
$app->post('/updatePrimaryAddress', 'App\Controllers\MyAccountController:updatePrimaryAddress');
$app->post('/addAddress', 'App\Controllers\MyAccountController:addAddress');
$app->post('/updateAddress', 'App\Controllers\MyAccountController:updateAddress');
$app->post('/deleteAddress', 'App\Controllers\MyAccountController:deleteAddress');
$app->post('/getBillDetail', 'App\Controllers\MyAccountController:getBillDetail');
$app->post('/getOrderHistory', 'App\Controllers\MyAccountController:getOrderHistory');

$app->post('/getFavouriteList', 'App\Controllers\MyAccountController:getFavouriteList');
$app->post('/getTableHistory', 'App\Controllers\MyAccountController:getTableHistory');
$app->post('/updateTblFKDomain', 'App\Controllers\MyAccountController:updateTblFKDomain');
$app->post('/orderTracking', 'App\Controllers\MyAccountController:orderTracking');
$app->post('/orderAgain', 'App\Controllers\MyAccountController:orderAgain');
$app->post('/createMyaccountFeedback', 'App\Controllers\MyAccountController:createMyaccountFeedback');

# stripe payment related api
$app->post('/walletDashboard', 'App\Controllers\PaymentController:walletDashboard');
$app->post('/getWalletHistory', 'App\Controllers\PaymentController:getWalletHistory');
$app->post('/getSaveCard', 'App\Controllers\PaymentController:getSaveCard');
$app->post('/updatePrimarySaveCard', 'App\Controllers\PaymentController:updatePrimarySaveCard');
$app->post('/deleteSaveCard', 'App\Controllers\PaymentController:deleteSaveCard');
$app->post('/addSaveCard', 'App\Controllers\PaymentController:addSaveCard');

# subscriptions related apis
$app->post('/webhook', 'App\Controllers\WebHookController:webhook');
$app->post('/createSubscription', 'App\Controllers\SubscriptionController:createSubscription');
$app->post('/getSubscriptionCoupon', 'App\Controllers\SubscriptionController:getSubscriptionCoupon');
$app->post('/unSubscription', 'App\Controllers\SubscriptionController:unSubscription');

#external api
$app->post('/leadSignup', 'App\Controllers\PagesController:leadSignup');
$app->post('/sendConfirm', 'App\Controllers\MyAccountController:orderAcceptMail');
$app->post('/orderReject', 'App\Controllers\MyAccountController:orderRejectMail');
$app->post('/orderaccept', 'App\Controllers\MyAccountController:orderAcceptMail');
$app->post('/orderdelivery', 'App\Controllers\MyAccountController:orderdelivery');
$app->post('/tableBookingRefund', 'App\Controllers\PaymentController:tableBookingRefund');
$app->post('/createEmailFeedback', 'App\Controllers\MenuController:createEmailFeedback');
$app->post('/sendOrderSms', 'App\Controllers\MyAccountController:sendOrderSms');
$app->post('/getClientFeedbackStatus', 'App\Controllers\MyAccountController:getClientFeedbackStatus');
$app->post('/getGoogleAddress', 'App\Controllers\PostcodeController:getGoogleAddress');

$app->get('/testing', 'App\Controllers\MyAccountController:testing');

# filter list page api
$app->post('/getRestaurantList', 'App\Controllers\FilterController:getRestaurantList');
$app->post('/getFilterType', 'App\Controllers\FilterController:getFilterType');

# couopn list api
$app->post('/getOfferDetailsList', 'App\Controllers\MenuController:getOfferDetailsList');
$app->post('/searchOfferCode', 'App\Controllers\MenuController:searchOfferCode');
# search postcode
$app->post('/searchPostcode', 'App\Controllers\AddressController:searchPostcode');
#Feedback Api
$app->post('/commonFeedback', 'App\Controllers\FeedbackController:commonFeedback');
$app->post('/getCommonFeedback', 'App\Controllers\FeedbackController:getCommonFeedback');

$app->post('/sendEposemail', 'App\Controllers\MyAccountController:sendEposemail'); // temporary line for Epos email
$app->post('/send-otp', 'App\Controllers\MyAccountController:send_otp');

# seo page api
$app->get('/sitemap', 'App\Controllers\SeoController:getSitemap');
$app->get('/getSeoData', 'App\Controllers\SeoController:getSeoData');
$app->POST('/inCompleteOrder', 'App\Controllers\PaymentController:inCompleteOrder');
$app->POST('/placeOrderIncompletePayment', 'App\Controllers\PaymentController:placeOrderIncompletePayment');


#client website api
$app->post('/businessType', 'App\Controllers\ClientHomeController:getbusinessType');
$app->post('/BannerSection', 'App\Controllers\ClientHomeController:getBannerSection');
$app->post('/MealDealSection', 'App\Controllers\ClientHomeController:getMealDeal');
$app->post('/AboutUsSection', 'App\Controllers\ClientHomeController:getAboutUs');
$app->post('/SignUpSection', 'App\Controllers\ClientHomeController:getSignUp');
$app->post('/GallerySection', 'App\Controllers\ClientHomeController:getGallery');
$app->post('/BookTableSection', 'App\Controllers\ClientHomeController:getBookTable');
$app->post('/ReviewSection', 'App\Controllers\ClientHomeController:getReview');
$app->post('/FeedbackSection', 'App\Controllers\ClientHomeController:getfeedback');
$app->post('/homePage', 'App\Controllers\ClientHomeController:index');
$app->post('/getMenuImage', 'App\Controllers\ClientHomeController:getMenuImage');
$app->post('/checkDomainWithPath', 'App\Controllers\ClientHomeController:checkDomainWithPath');
$app->post('/getSocialLink', 'App\Controllers\ClientHomeController:getSocialLink');
$app->post('/getOurBranchSection', 'App\Controllers\ClientHomeController:getOurBranchSection');
$app->post('/getGalleryGridSection', 'App\Controllers\ClientHomeController:getGalleryGridSection');
$app->post('/getAdditionalButtonSection', 'App\Controllers\ClientHomeController:getAdditionalButtonSection');



#Seo Controller
$app->post('/sitemapClient', 'App\Controllers\ClientSeoController:sitemap');
$app->post('/getSeoDataClient', 'App\Controllers\ClientSeoController:getSeoData');

#Feedback Controller
$app->post('/feedbackSubmit', 'App\Controllers\FeedbackController:getFeedbackData');

#BookTable Controller
$app->post('/CheckTableAvaliablity', 'App\Controllers\BookTableController:CheckTableAvaliablity');
$app->post('/bookTableForNow', 'App\Controllers\BookTableController:bookTableForNow');
$app->post('/getTableTiming', 'App\Controllers\BookTableController:getTableTiming');
$app->post('/gettablegallery', 'App\Controllers\ClientHomeController:gettablegallery');

#getPromotion
$app->post('/getPromotionSection', 'App\Controllers\ClientHomeController:getPromotionSection');

# menu action api
$app->post('/getCategories', 'App\Controllers\MenuClientActionController:getCategories');
$app->post('/getCategoryById', 'App\Controllers\MenuClientActionController:getCategoryById');
$app->post('/insertCategory', 'App\Controllers\MenuClientActionController:createCategory');
$app->post('/editCategory', 'App\Controllers\MenuClientActionController:editCategory');
$app->post('/deleteCategory', 'App\Controllers\MenuClientActionController:deleteCategory');
$app->post('/getSubgroupbyCat', 'App\Controllers\MenuClientActionController:getSubgroupbyCat');

$app->post('/getSubCategories', 'App\Controllers\MenuClientActionController:getSubCategories');
$app->post('/getSubCategoryById', 'App\Controllers\MenuClientActionController:getSubCategoryById');
$app->post('/insertSubCategery', 'App\Controllers\MenuClientActionController:createSubCategery');
$app->post('/editSubCategory', 'App\Controllers\MenuClientActionController:editSubCategory');
$app->post('/deleteSubCategory', 'App\Controllers\MenuClientActionController:deleteSubCategory');

$app->post('/getAllItems', 'App\Controllers\MenuClientActionController:getAllItems');
$app->post('/getItemsById', 'App\Controllers\MenuClientActionController:getItemsById');
$app->post('/insertItem', 'App\Controllers\MenuClientActionController:createItems');
$app->post('/editItems', 'App\Controllers\MenuClientActionController:editItems');
$app->post('/deleteItem', 'App\Controllers\MenuClientActionController:deleteItem');

$app->post('/getAllAddons', 'App\Controllers\MenuClientActionController:getAllAddons');
$app->post('/getAddonById', 'App\Controllers\MenuClientActionController:getAddonById');
$app->post('/getAllAddonItemsByAddonId', 'App\Controllers\MenuClientActionController:getAllAddonItemsByAddonId');
$app->post('/insertAddon', 'App\Controllers\MenuClientActionController:createAddon');
$app->post('/editAddon', 'App\Controllers\MenuClientActionController:editAddon');
$app->post('/deleteAddon', 'App\Controllers\MenuClientActionController:deleteAddon');

$app->post('/getAllAddonItems', 'App\Controllers\MenuClientActionController:getAllAddonItems');
$app->post('/getAddonItemById', 'App\Controllers\MenuClientActionController:getAddonItemById');
$app->post('/insertAddonItem', 'App\Controllers\MenuClientActionController:createAddonItem');
$app->post('/editAddonItem', 'App\Controllers\MenuClientActionController:editAddonItem');
$app->post('/updateStatus', 'App\Controllers\MenuClientActionController:updateStatus');
$app->post('/deleteAddonItem', 'App\Controllers\MenuClientActionController:deleteAddonItem');

$app->post('/updatingMenu', 'App\Controllers\MenuClientActionController:updatingAllMenu');
$app->post('/moveMenuToFK', 'App\Controllers\MenuClientActionController:moveMenuToFK');

$app->run();
